﻿using System;
using System.Diagnostics;
using System.Drawing;
using System.Windows.Forms;

namespace Campo_Minato
{
    public partial class Form1 : Form
    {
        #region Dichiarazioni pubbliche

        internal struct S // la scacchiera ha le seguenti info
        {
            public byte nRighe; // numero di righe
            public byte nColonne; // numero di colonne
            public byte wQuadratino; // larghezza del quadratino
            public byte hQuadratino; // altezza del quadratino
            public bool morto; // se il gioco è finito male
        }

        internal struct C // ogni casella contenuta in schema(,) nel tag ha le info seguenti
        {
            public byte indCol; // indice di colonna
            public byte indRig; // indice di riga
            public byte imgQuadratino; // quale immagine è visualizzata (0=quadratino, 1=bandierina, 2=interrogativo)
        }

        private PictureBox[,] schema; // matrice bidimensionale con le picturebox
        private S Scacchiera; // contiene le info della scacchiera
        private C indici; // contiene le info di ogni quadratino
        private bool ricorsivo = false; // attiva la ricorsività
        private Int16 nMine = 0; // si decrementa ad ogni bandierina o interrogativo messo giusto o sbagliato che sia
        private UInt16 nMineTrovare = 0; // nMineTrovare non cambierà più per tutto il gioco in corso
        private UInt16 nMineTrovate = 0; // si incrementa quando la bandierina è su una mina
        private byte OffsetL = 3; // posizione scacchiera nel panel
        private byte OffsetT = 3; // posizione scacchiera nel panel
        private Stopwatch sw = new Stopwatch(); // orologio contatempo
        private byte imgQuadratino = 0; // 0=quaratino coprente (gioco normale), 3=quadratino trasparente (debug)

        #endregion // fine dichiarazioni pubbliche

        public Form1()
        {
            InitializeComponent();
        }

        #region creazione gioco
 
        private void Form1_Load(object sender, EventArgs e)
        {
            PanelSmile_Click(PanelSmile, new EventArgs()); // simula il click nel PanelSmile
            ComboBox1.SelectedIndex = 3; // il cambio indice scatena il riempimento dello schema
        }

        private void ComboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            TxtC.Text = Scacchiera.nColonne.ToString();
            TxtR.Text = Scacchiera.nRighe.ToString();
            TxtSize.Text = Scacchiera.hQuadratino.ToString();
            popolaSchema();
        }

        private void PanelSmile_Click(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;
            ButRipeti.Visible = false;
            Timer1.Enabled = false;
            this.SuspendLayout();

            byte vecchierighe = Scacchiera.nRighe;
            byte vecchiecolonne = Scacchiera.nColonne;
            byte vecchiosize = Scacchiera.wQuadratino;

            // controllo conformità parametri sulle textbox
            if (byte.TryParse(TxtSize.Text, out Scacchiera.hQuadratino) == false) Scacchiera.hQuadratino = 30;
            if (byte.TryParse(TxtSize.Text, out Scacchiera.wQuadratino) == false) Scacchiera.wQuadratino = 30;
            if (byte.TryParse(TxtC.Text, out Scacchiera.nColonne) == false) Scacchiera.nColonne = 10;
            if (byte.TryParse(TxtR.Text, out Scacchiera.nRighe) == false) Scacchiera.nRighe = 10;

            // controllo valori eccessivi sulle textbox
            if (Scacchiera.nColonne * Scacchiera.nRighe > 2500)
            {
                if (Scacchiera.nColonne > Scacchiera.nRighe) Scacchiera.nColonne = Convert.ToByte(2500 / Scacchiera.nRighe);
                else Scacchiera.nRighe = Convert.ToByte(2500 / Scacchiera.nColonne);
            }
            if (Scacchiera.hQuadratino < 15)
            {
                Scacchiera.hQuadratino = 15;
                Scacchiera.wQuadratino = 15;
            }
            if (Scacchiera.hQuadratino > 60)
            {
                Scacchiera.hQuadratino = 60;
                Scacchiera.wQuadratino = 60;
            }

            // visualizzazione parametri normalizzati
            TxtC.Text = Scacchiera.nColonne.ToString("00");
            TxtR.Text = Scacchiera.nRighe.ToString("00");
            TxtSize.Text = Scacchiera.hQuadratino.ToString("00");

            PanelSmile.BackgroundImage = Properties.Resources.Smile;

            if (vecchiecolonne == Scacchiera.nColonne && vecchierighe == Scacchiera.nRighe)
            {
                // se le righe e le colonne dello schema non sono cambiate
                // inutile ricrearlo, è sufficiente ripopolarlo
                popolaSchema();
                if (vecchiosize != Scacchiera.wQuadratino) // solo se il size è cambiato
                {
                    ridimensionaSchema();
                }
            }
            else
            {
                // le righe e/o le colonne sono cambiate
                if (Panel3.Controls.Count != 0) // se non è la prima partita svuoto il panel
                {
                    Panel3.Controls.Clear();
                }
                schema = new PictureBox[Scacchiera.nColonne, Scacchiera.nRighe]; // nuova matrice vuota
                creaSchema(); // crea la matrice bidimensionale di picturebox e posiziona in panel3
                popolaSchema(); // crea e conta le mine intorno ad ogni casella e ne scrive il numero
            }

            this.ResumeLayout();
            this.PerformLayout();
            Application.DoEvents();
            Form1_ResizeEnd(this, new EventArgs());
            Panel3.BorderStyle = BorderStyle.FixedSingle;
            LabTempo.Focus(); // tolgo il fuoco alle textbox se ce l'avessero
            this.Cursor = Cursors.Default;
        }

        private void creaSchema()
        {
            for (byte riga = 0; riga < Scacchiera.nRighe; riga++)
            {
                for (byte colonna = 0; colonna < Scacchiera.nColonne; colonna++)
                {
                    schema[colonna, riga] = new PictureBox();
                    schema[colonna, riga].Name = "";
                    schema[colonna, riga].SizeMode = PictureBoxSizeMode.StretchImage;
                    schema[colonna, riga].BackgroundImageLayout = ImageLayout.Stretch;
                    schema[colonna, riga].Size = new Size(Scacchiera.wQuadratino - 1, Scacchiera.hQuadratino - 1);
                    schema[colonna, riga].Location = new Point(colonna * Scacchiera.wQuadratino + OffsetL, riga * Scacchiera.hQuadratino + OffsetT);
                    schema[colonna, riga].MouseClick += new MouseEventHandler(schema_click);
                    schema[colonna, riga].MouseDown += new MouseEventHandler(schema_MouseDown);
                    schema[colonna, riga].MouseUp += new MouseEventHandler(schema_MouseUp);
                    Panel3.Controls.Add(schema[colonna, riga]);
                }
            }
        }

        private void popolaSchema()
        {
            if (ComboBox1.SelectedIndex == -1)  return;
            LabTempo.Text = "____";
            LabMine.Text = "___";
            Application.DoEvents();
            byte percentuale = Convert.ToByte(ComboBox1.SelectedIndex + 1); // da 1 = 60% a 9 = 10% (circa)
            nMine = 0; // vengono contate tutte le mine e poi si decrementa con le bandierine, anche se messe sbagliate
            UInt16 nCaselle = Convert.ToUInt16(Scacchiera.nColonne * Scacchiera.nRighe);
            Random rnd = new Random();
            this.Cursor = Cursors.WaitCursor;
            this.SuspendLayout();
            Panel3.BackColor = this.BackColor;
            Panel3.Enabled = true;
            Scacchiera.morto = false;
            LabTempo.ForeColor = Color.Yellow;
            PanelSmile.BackgroundImage = Properties.Resources.Smile;

            for (byte riga = 0; riga < Scacchiera.nRighe; riga++)
            {
                for (byte colonna = 0; colonna < Scacchiera.nColonne; colonna++)
                {
                    schema[colonna, riga].BackgroundImage = null;
                    schema[colonna, riga].BackColor = Color.DarkGray;
                    schema[colonna, riga].Name = "";
                    indici.indCol = Convert.ToByte(colonna);
                    indici.indRig = Convert.ToByte(riga);
                    indici.imgQuadratino = 0;
                    schema[colonna, riga].Tag = indici;
                    if (Math.Truncate(rnd.Next(1 * percentuale + 1) / (double)percentuale) == 1) // solo gli uno sono promossi a mina
                    {
                        schema[colonna, riga].BackgroundImage = Properties.Resources.mina;
                        schema[colonna, riga].Name = "mina";
                        nMine += 1; // conteggio mine
                    }
                    schema[colonna, riga].Image = ImageList1.Images[imgQuadratino];
                }
            }
            LabMine.Text = nMine.ToString("000");
            nMineTrovare = Convert.ToUInt16(nMine); // nMineTrovare non cambierà più per tutto il gioco in corso, invece nMine si decrementa con le bandierine
            nMineTrovate = 0; // si incrementa con la bandierina, se messa su una mina

            // ciclo di tutte le caselle
            for (byte riga = 0; riga < Scacchiera.nRighe; riga++)
            {
                for (byte colonna = 0; colonna < Scacchiera.nColonne; colonna++)
                {
                    if (schema[colonna, riga].Name != "mina")
                    {
                        // ciclo che conta le mine intorno alla casella, se non è una mina
                        UInt16 quanteMine = 0;

                        for (sbyte r = Convert.ToSByte(riga - 1); r <= Convert.ToSByte(riga + 1); r++)
                        {
                            if (r >= 0 && r < Scacchiera.nRighe)
                            {
                                for (sbyte c = Convert.ToSByte(colonna - 1); c <= Convert.ToSByte(colonna + 1); c++)
                                {
                                    if (c >= 0 && c < Scacchiera.nColonne)
                                    {
                                        if (schema[c, r].Name == "mina") quanteMine += 1;
                                    }
                                }
                            }
                        }

                        schema[colonna, riga].Name = quanteMine.ToString(); // sul nome il numero delle mine che lo circonda, "0" è usato per la ricerca ricorsiva
                        Brush colore = Brushes.Black;
                        if (quanteMine == 0) colore = new SolidBrush(schema[colonna, riga].BackColor); // stesso colore = invisibile
                        if (quanteMine == 1) colore = Brushes.Blue;
                        if (quanteMine == 2) colore = Brushes.DarkGreen;
                        if (quanteMine == 3) colore = Brushes.Red;
                        if (quanteMine == 4) colore = Brushes.Orange;
                        if (quanteMine == 5) colore = Brushes.Bisque;
                        if (quanteMine == 6) colore = Brushes.DarkViolet;
                        if (quanteMine == 7) colore = Brushes.Firebrick;

                        Bitmap bmp = new Bitmap(Scacchiera.wQuadratino, Scacchiera.hQuadratino);
                        Graphics g = Graphics.FromImage(bmp);
                        g.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
                        g.DrawString(quanteMine.ToString(), new Font("Courier New", Convert.ToInt16(Scacchiera.hQuadratino / 1.5), FontStyle.Bold), colore, 4, 0);
                        schema[colonna, riga].BackgroundImage = bmp;
                        g.Dispose();
                    }
                }
            }
            sw.Restart(); // partenza orologio
            this.ResumeLayout();
            Form1_ResizeEnd(this, new EventArgs());
            this.Cursor = Cursors.Default;
            Timer1.Enabled = true; // visualizzazione orologio
        }

        private void ButRipeti_Click(object sender, EventArgs e)
        {
            C parametri;
            LabTempo.Text = "____";
            LabMine.Text = "___";
            LabTempo.ForeColor = Color.Yellow;
            LabMine.ForeColor = Color.Lime;
            ButRipeti.Visible = false;
            Application.DoEvents();
            // ciclo di tutte le caselle per ripristinare tutti i vecchi valori
            for (var riga = 0; riga < Scacchiera.nRighe; riga++)
            {
                for (byte colonna = 0; colonna < Scacchiera.nColonne; colonna++)
                {
                    schema[colonna, riga].Image = ImageList1.Images[imgQuadratino];
                    parametri = (C)schema[colonna, riga].Tag;
                    parametri.imgQuadratino = 0;
                    schema[colonna, riga].Tag = parametri;
                    if (schema[colonna, riga].Name != "mina")
                    {
                        byte quantemine = Convert.ToByte(schema[colonna, riga].Name); // recupero il numero delle mine
                        Brush colore = Brushes.Black;
                        if (quantemine == 0) colore = new SolidBrush(schema[colonna, riga].BackColor); // stesso colore = invisibile
                        if (quantemine == 1) colore = Brushes.Blue;
                        if (quantemine == 2) colore = Brushes.DarkGreen;
                        if (quantemine == 3) colore = Brushes.Red;
                        if (quantemine == 4) colore = Brushes.Orange;
                        if (quantemine == 5) colore = Brushes.Bisque;
                        if (quantemine == 6) colore = Brushes.DarkViolet;
                        if (quantemine == 7) colore = Brushes.Firebrick;

                        Bitmap bmp = new Bitmap(Scacchiera.wQuadratino, Scacchiera.hQuadratino);
                        Graphics g = Graphics.FromImage(bmp);
                        g.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
                        g.DrawString(quantemine.ToString(), new Font("Courier New", Convert.ToInt16(Scacchiera.hQuadratino / 1.5), FontStyle.Bold), colore, 4, 0);
                        schema[colonna, riga].BackgroundImage = bmp;
                        g.Dispose();
                    }
                    else
                    {
                        schema[colonna, riga].BackColor = Color.DarkGray;
                        schema[colonna, riga].BackgroundImage = Properties.Resources.mina;
                    }
                }
            }
            sw.Restart();
            Panel3.BackColor = this.BackColor;
            Scacchiera.morto = false;
            LabTempo.ForeColor = Color.Yellow;
            PanelSmile.BackgroundImage = Properties.Resources.Smile;
            Panel3.Enabled = true;
            nMineTrovate = 0;
            nMine = Convert.ToInt16(nMineTrovare);
            LabMine.Text = nMine.ToString("000");
            Timer1.Enabled = true;
        }

        private void Panel2_MouseEnter(object sender, EventArgs e)
        {
            PanelSmile.BorderStyle = BorderStyle.Fixed3D;
        }

        private void Panel2_MouseLeave(object sender, EventArgs e)
        {
            PanelSmile.BorderStyle = BorderStyle.None;
        }

        private void ButPiu_Click(object sender, EventArgs e)
        {
            Scacchiera.hQuadratino += 1;
            if (Scacchiera.hQuadratino > 60)
            {
                Scacchiera.hQuadratino = 60;
            }
            TxtSize.Text = Scacchiera.hQuadratino.ToString("00");
            Scacchiera.wQuadratino = Scacchiera.hQuadratino;
            ridimensionaSchema();
        }

        private void ButMeno_Click(object sender, EventArgs e)
        {
            Scacchiera.hQuadratino -= 1;
            if (Scacchiera.hQuadratino < 15)
            {
                Scacchiera.hQuadratino = 15;
            }
            TxtSize.Text = Scacchiera.hQuadratino.ToString("00");
            Scacchiera.wQuadratino = Scacchiera.hQuadratino;
            ridimensionaSchema();
        }

        private void ridimensionaSchema()
        {
            this.Cursor = Cursors.WaitCursor;
            this.SuspendLayout();
            for (var riga = 0; riga < Scacchiera.nRighe; riga++)
            {
                for (var colonna = 0; colonna < Scacchiera.nColonne; colonna++)
                {
                    schema[colonna, riga].Size = new Size(Scacchiera.wQuadratino - 1, Scacchiera.hQuadratino - 1);
                    schema[colonna, riga].Location = new Point(colonna * Scacchiera.wQuadratino + OffsetL, riga * Scacchiera.hQuadratino + OffsetT);
                }
            }
            this.ResumeLayout();
            Form1_ResizeEnd(this, new EventArgs());
            this.Cursor = Cursors.Default;
        }

        private void Txt_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Return)
            {
                PanelSmile_Click(PanelSmile, new EventArgs());
            }
        }

        #endregion // fine creazione gioco

        #region Controllo gioco

        private void schema_click(object sender, MouseEventArgs e)
        {
            PictureBox pb = (PictureBox)sender;
            C ind = (C)pb.Tag;
            if (e.Button == MouseButtons.Right)
            {
                if ((pb.Image == null)) // è scoperta
                {
                    return;
                }
                ind.imgQuadratino += 1; // ciclo le immagini 0, 1, 2
                if (ind.imgQuadratino == 3) ind.imgQuadratino = 0; // ricomincio il ciclo

                pb.Image = ImageList1.Images[ind.imgQuadratino]; // mostro l'immagine associata, 0=vuoto, 1=bandierina, 2=interrogativo
                pb.Tag = ind; // aggiorno il parametro

                if (ind.imgQuadratino == 1) nMine -= 1;
                if (ind.imgQuadratino == 0) nMine += 1;
                if (ind.imgQuadratino == 1 && pb.Name == "mina") nMineTrovate += 1;
                if (ind.imgQuadratino == 0 && pb.Name == "mina") nMineTrovate -= 1;
                if (nMine > -1) LabMine.ForeColor = Color.Lime;
                else LabMine.ForeColor = Color.Red;

                LabMine.Text = nMine.ToString("000");
                return;
            }
            else if (e.Button == MouseButtons.Left && ind.imgQuadratino == 0) scopri(ind.indCol, ind.indRig); // click sinistro e immagine sul quadratino assente
        }

        private void schema_MouseDown(object sender, MouseEventArgs e)
        {
            PanelSmile.BackgroundImage = Properties.Resources.Oh;
        }

        private void schema_MouseUp(object sender, MouseEventArgs e)
        {
            // controllo fine gioco
            if (nMineTrovate == nMineTrovare && nMine == 0) // schema risolto
            {
                PanelSmile.BackgroundImage = Properties.Resources.sunglass;
                // tutte le mine sono state individuate
                for (var riga = 0; riga < Scacchiera.nRighe; riga++)
                {
                    for (var colonna = 0; colonna < Scacchiera.nColonne; colonna++)
                    {
                        C info = (C)schema[colonna, riga].Tag;
                        if (schema[colonna, riga].Name == "mina")
                        {
                            schema[colonna, riga].BackColor = Color.Lime;
                            schema[colonna, riga].Image = null;
                        }
                    }
                }
                Panel3.Enabled = false;
                Panel3.BackColor = Color.DarkGreen;
                LabTempo.ForeColor = Color.Lime;
                Timer1.Enabled = false;
                ButRipeti.Visible = true;
            }
            else
            {
                if (Scacchiera.morto) // schema non risolto
                {
                    PanelSmile.BackgroundImage = Properties.Resources.Die;
                    LabTempo.ForeColor = Color.Red;
                    Timer1.Enabled = false;
                    ButRipeti.Visible = true;
                }
                else
                {
                    PanelSmile.BackgroundImage = Properties.Resources.Smile; // proseguo gioco
                }
            }
        }

        private void scopri(byte colonna, byte riga)
        {
            if (schema[colonna, riga].Name == "mina")
            {
                // gioco finito non riuscito
                schema[colonna, riga].BackColor = Color.Red;
                Panel3.BackColor = Color.DarkRed;
                Panel3.Enabled = false;
                UInt16 errori = 0;
                for (riga = 0; riga < Scacchiera.nRighe; riga++)
                {
                    for (colonna = 0; colonna < Scacchiera.nColonne; colonna++)
                    {
                        C info = (C)schema[colonna, riga].Tag;
                        // se era stata messa una bandierina dove non c'era una mina lo segnalo
                        if (info.imgQuadratino == 1 && schema[colonna, riga].Name != "mina" & !(schema[colonna, riga].Image == null))
                        {
                            schema[colonna, riga].BackgroundImage = Properties.Resources.minabarrata;
                            schema[colonna, riga].Image = null;
                            errori += 1;
                        }
                        // scopro tutte le mine
                        if (schema[colonna, riga].Name == "mina")
                        {
                            schema[colonna, riga].Image = null;
                        }
                    }
                }
                Scacchiera.morto = true;
            }
            else
            {
                if (RadioLogico.Checked) // con logico, tutte le caselle confinanti vengono aperte
                {
                    if (schema[colonna, riga].Image == null) // controllo se la casella è coperta
                    {
                        if (ricorsivo == false) return; // se si è al click su una casella scoperta si esce, se si è in ricorsione no
                    }
                    else schema[colonna, riga].Image = null; // la casella cliccata coperta viene scoperta

                    for (sbyte r = Convert.ToSByte(riga - 1); r <= Convert.ToSByte(riga + 1); r++)
                    {
                        if (r >= 0 && r < Scacchiera.nRighe)
                        {
                            for (sbyte c = Convert.ToSByte(colonna - 1); c <= Convert.ToSByte(colonna + 1); c++)
                            {
                                if (c >= 0 && c < Scacchiera.nColonne)
                                {
                                    controlloAdiacentiLogico(Convert.ToByte(c), Convert.ToByte(r));
                                }
                            }
                        }
                    }
                    ricorsivo = false;
                }
                else // con classico, solo se si clicca una casella vuota si aprono le caselle adiacenti
                {
                    schema[colonna, riga].Image = null; // la casella cliccata viene scoperta
                                                        // se si clicca una casella vuota a rotazione controllo tutte le caselle adiacenti, evitando di andare fuori schema
                    if (schema[colonna, riga].Name == "0")
                    {
                        for (sbyte r = Convert.ToSByte(riga - 1); r <= Convert.ToSByte(riga + 1); r++)
                        {
                            if (r >= 0 && r < Scacchiera.nRighe)
                            {
                                for (sbyte c = Convert.ToSByte(colonna - 1); c <= Convert.ToSByte(colonna + 1); c++)
                                {
                                    if (c >= 0 && c < Scacchiera.nColonne)
                                    {
                                        controlloAdiacentiClassico(Convert.ToByte(c), Convert.ToByte(r));
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        private void controlloAdiacentiClassico(byte colonna, byte riga)
        {
            // la casella non sarà mai una mina, perché le caselle con zero non hanno mine adiacenti
            // in ricorsione vengono controllate anche tutte le caselle intorno alla casella che contengono zero.
            // se è contrassegnata con una bandierina o ? la casella non viene scoperta
            C info = (C)schema[colonna, riga].Tag;

            if (info.imgQuadratino == 0) // non è contrassegnata
            {
                if (!(schema[colonna, riga].Image == null)) // la casella è coperta, la scopro
                {
                    schema[colonna, riga].Image = null;
                    // se la casella contiene zero (nessuna mina adiacente) controllo anche intorno a lei ricorsivamente
                    if (schema[colonna, riga].Name == "0")
                    {
                        scopri(colonna, riga);
                    }
                }
            }
        }

        private void controlloAdiacentiLogico(byte colonna, byte riga)
        {
            // se la casella non è una mina viene scoperta, se non era già scoperta
            // se non era una mina, non era scoperta e contiene zero (nessuna casella adiacente con mina)
            // in ricorsione vengono controllate anche tutte le caselle intorno alla casella che contiene zero.
            // se c'è una bandierina o ? la casella non viene scoperta
            C info = (C)schema[colonna, riga].Tag;
            if (schema[colonna, riga].Name != "mina" && info.imgQuadratino == 0)
            {
                // non è una mina e non è contrassegnata
                if (!(schema[colonna, riga].Image == null))
                {
                    // la casella è coperta, la scopro
                    schema[colonna, riga].Image = null;
                    // se la casella contiene zero (nessuna mina adiacente) controllo anche intorno a lei ricorsivamente
                    if (schema[colonna, riga].Name == "0")
                    {
                        ricorsivo = true;
                        scopri(colonna, riga);
                    }
                }
            }
        }
        
        private void Timer1_Tick(object sender, EventArgs e)
        {
            LabTempo.Text = (sw.ElapsedMilliseconds / 1000).ToString("0000");
        }

        #endregion // fine Controllo Gioco


        #region Controllo posizioni nel form

        private void Form1_ResizeEnd(object sender, EventArgs e)
        {
            Application.DoEvents();
            int p3left = Convert.ToInt32(this.Width / 2.0 - Panel3.Width / 2.0 - 10); // centrato
            int p3Width = Scacchiera.wQuadratino * (Scacchiera.nColonne) + OffsetT * 2 + 1;
            if (p3left < 3 || p3Width > Screen.PrimaryScreen.Bounds.Width) p3left = 3; // se la centratura porta lo schema fuori a sinistra, lo evito
            Panel3.Left = p3left; // centrato
            Panel3.Width = p3Width;
            Panel3.Height = Scacchiera.hQuadratino * (Scacchiera.nRighe) + OffsetL * 2 + 1;
            Panel1.Left = this.Width / 2 - Panel1.Width / 2 - 8; // centrato
        }

        private void Form1_SizeChanged(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Maximized)
            {
                int p3left = Convert.ToInt32(this.Width / 2.0 - Panel3.Width / 2.0 - 10); // centrato
                int p3Width = Scacchiera.wQuadratino * (Scacchiera.nColonne) + OffsetT * 2 + 1;
                if (p3left < 3 || p3Width > Screen.PrimaryScreen.Bounds.Width) p3left = 3; // se la centratura porta lo schema fuori a sinistra, lo evito
                Panel3.Left = p3left; // centrato
                Panel3.Width = p3Width;
                Panel3.Height = Scacchiera.hQuadratino * (Scacchiera.nRighe) + OffsetL * 2 + 1;
                Panel1.Left = this.Width / 2 - Panel1.Width / 2 - 8; // centrato
            }
        }

        #endregion // fine Controllo posizioni form
    }
}

